/**
 * \file SDL_conf.h -- function declarations for config file management
 *
 *   SDL_conf is a system for management of external application data.
 * There are two kinds of data:  static and dynamic.
 *
 *   Static data is for things like images that aren't going to change;  
 * these are to be  stored in a directory like /usr/share/appname/, etc.
 *
 *   Dynamic data is data that may change between program invocations 
 * and indeed may change while the program is running;  these are stored 
 * under ~/.appname/persistent.xml
 *
 *   The dynamic data is stored hierarchially in an XML file.  All tags 
 * are objects, directories, or both;  objects contain attributes like
 * x="5", directories contain further tags.  All information is stored
 * in attributes, character data between tags is ignored.  Other than
 * these restrictions you may organize the data in any way you see fit;
 * SDL_conf is just a data storage system and does not care what you do
 * with it.
 */
#ifndef __SDL_CONF_H__
#define __SDL_CONF_H__

#include <SDL/begin_code.h>

#ifdef __cplusplus
extern "C" {
#endif

#define TRUE_STR  "true"
#define FALSE_STR "false"

#define CONF_CREATE   0x0001
#define CONF_AUTOSAVE 0x0002

#define BAD_LONG (~0L)
extern const float BAD_FLOAT; /* NAN */

/*! \brief Loads configuration data */
DECLSPEC int SDLCALL SDL_ConfLoad(const char *app, int flags);
/*! \brief Saves configuration data */
DECLSPEC int SDLCALL SDL_ConfSave();
/*! \brief Closes configuration handle */
DECLSPEC void SDLCALL SDL_ConfClose();
/*! \brief Compacts the tree, getting rid of freed nodes. Will invalidate seeks.*/
DECLSPEC int SDLCALL SDL_ConfCompact();
/*! \brief Get current static path */
DECLSPEC const char * SDLCALL SDL_ConfStaticPath();

/*! \brief Returns nonzero if tag exists, zero otherwise */
DECLSPEC int SDLCALL SDL_ConfTagExists(void *root, const char *name);
/*! \brief Seeks partly down the global data tree */
DECLSPEC void *SDL_ConfSeekNext(void *prev, const char *path, int create);
/*! \brief Helper macro for new seeks */
#define SDL_ConfSeek(path,create) SDL_ConfSeekNext(NULL,path,create)
/*! \brief Attempts to remove a tag and all attributes below it.  Cannot remove the config root. */
DECLSPEC int SDLCALL SDL_ConfTagRemove(void *root);

/*! \brief Sets an attribute */
DECLSPEC int SDLCALL SDL_ConfSet(void *root, const char *att, const char *val);
/*! \brief Retrieves an attribute, if any */
DECLSPEC const char *SDL_ConfGet(void *root, const char *att);
/*! \brief Removes an attribute from the tag. */
#define SDL_ConfAttClear(root,att) SDL_ConfSet(root,att,NULL)

/*! \brief Sets a long-value attribute */
DECLSPEC int SDLCALL SDL_ConfSetLong(void *root, const char *att, long val);
/*! \brief Retrieves long-val attributes with specified default */
DECLSPEC long SDLCALL SDL_ConfGetLongDefault(void *root, const char *att, long def);
/*! \brief Helper-macro to retrieve long values with default of BAD_LONG */
#define SDL_ConfGetLong(r,a) SDL_ConfGetLongDefault(r,a,BAD_LONG)

/*! \brief Helper function to set a floating-point attribute */
DECLSPEC int SDLCALL SDL_ConfSetFloat(void *root, const char *att, float val);
/*! \brief Helper functioin to read a floating-point attribute */
DECLSPEC float SDLCALL SDL_ConfGetFloatDefault(void *root, const char *att, float def);
/*! \brief Floating-point read macro that defaults to BAD_FLOAT */
#define SDL_ConfGetFloat(r,a) SDL_ConfGetFloatDefault(r,a,BAD_FLOAT)

/*! \brief Helper function to set a boolean attribute */
DECLSPEC int SDLCALL SDL_ConfSetBool(void *root, const char *att, int bval);
/*! \brief Helper function to read a boolean attribute */
DECLSPEC int SDLCALL SDL_ConfGetBoolDefault(void *root, const char *att, int bdef);
/*! \brief Boolean read macro that defaults to false */
#define SDL_ConfGetBool(r,a) SDL_ConfGetBoolDefault(r,a,0)
/*! \brief Helper function that sets only if doesn't already exist */
DECLSPEC int SDLCALL SDL_ConfSetDefault(void *root, const char *att, const char *val);
#ifdef __cplusplus
}
#endif

#include <SDL/close_code.h>

#endif/*__SDL_CONF_H__*/
