/* system-specific headers */
#if defined(unix)||defined(__unix__)||defined(_POSIX_SOURCE)

#define UNIX_PATHS
#define DIR_SEP_STR "/"
#define DIR_SEP_CHAR '/'

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#define CREATE_DIR(path) ( mkdir(path,0700)==0)
  static int TEST_DIR(const char *path)
  {
    struct stat s;
    if(stat(path,&s)!=0)    return(0);
    if(!S_ISDIR(s.st_mode)) return(0);
    return(1);
  }

#elif defined(WIN32_WCE)
#define WCE_PATHS
#elif defined(WIN32)

#define WIN32_LEAN_AND_MEAN
#define DIR_SEP_STR  "\\"
#define DIR_SEP_CHAR '\\'
#define WIN32_PATHS
#define INVALID_FILE_ATTRIBUTES 0xFFFFFFFF
#define CREATE_DIR(path) (CreateDirectory(path,0L)!=0)

#include <windows.h>

  static int TEST_DIR(const char *path)
  {
    int result=GetFileAttributes(path);
    if(result == INVALID_FILE_ATTRIBUTES) return(0);
    return(result & FILE_ATTRIBUTE_DIRECTORY);
  }

#endif/*WIN32*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL_getenv.h>

#include "SDL_datadir.h"

/*! \brief Dynamic data paths -- per user config files etc */
extern const char *dynamic_data_path[];
/*! \brief Static data paths -- global read-only data */
extern const char *static_data_path[];

#ifdef DATADIR_MAIN_TEST
int main(int argc, char *argv[])
{
  char buf[512];
  char *retval=SDL_DirStatic(buf,argv[0],512);

  SDL_DirPrint();

  if(retval==NULL)
    fprintf(stderr,"Couldn't find static data dir\n");
  else
    fprintf(stderr,"Found static data dir '%s'\n",buf);

  retval=SDL_DirDynamic(buf,argv[0],512,1);
  if(retval==NULL)
    fprintf(stderr,"Couldn't find dynamic data dir\n");
  else
    fprintf(stderr,"Found dynamic data dir '%s'\n",buf);

  return(0);
}
#endif

char *SDL_DirDynamic(char *bufout, const char *app, int maxlen, int make)
{
#ifdef UNIX_PATHS
	char buf[512]=".";

#else
	char buf[512]="";
#endif

  {
    const char *tmp=strchr(app,DIR_SEP_CHAR);
    while(tmp!=NULL)
    {
      app=tmp+1;
      tmp=strchr(app,DIR_SEP_CHAR);
    }
  }

  strcat(buf,app);
  return(SDL_DirData(dynamic_data_path,bufout,buf,maxlen,make));
}

char *SDL_DirStatic(char *bufout, const char *app, int maxlen)
{
  return(SDL_DirData(static_data_path,bufout,app,maxlen,0));
}

char *SDL_DirData(const char *path[], char *bufout, const char *app, 
                  int maxlen, int mksubdir)
{
  char buf[512];
  int n;

  if((bufout==NULL)||(maxlen<=0)) return(NULL);

  if(app!=NULL)
  {
    const char *tmp=strchr(app,DIR_SEP_CHAR);
    while(tmp!=NULL)
    {
      app=tmp+1;
      tmp=strchr(app,DIR_SEP_CHAR);
    }
  }

#ifdef WIN32
  if(getenv("username") == NULL)
  {
    int size;
    strcpy(buf,"username=");
    GetUserName(buf+strlen(buf),&size);
    putenv(buf);
    buf[0]='\0'    
  }
#endif

  for(n=0; path[n]!=NULL; n++)
  {
    int ipos=0,opos=0;
	for(ipos=0; path[n][ipos] != '\0'; ipos++)
	{
	  if(path[n][ipos]=='$')
	  {
		int tpos=0;
		char tbuf[256];
		ipos++;

		while(isalnum(path[n][ipos])||isspace(path[n][ipos])||(path[n][ipos]=='_'))
          tbuf[tpos++]=path[n][ipos++];

		ipos--;

		tbuf[tpos++]='\0';
		if(getenv(tbuf)==NULL)
		{
		  fprintf(stderr,"Couldn't getenv for %s\n",tbuf);
		  goto DIR_FAIL;
		}

		strcpy(buf+opos,getenv(tbuf));
		opos=strlen(buf);
	  }
	  else
	    buf[opos++]=path[n][ipos];
	}

    buf[opos++]='\0';

    if(app==NULL)
      goto RETURN_OK;

    strcat(buf,app);

    fprintf(stderr,"Testing subpath '%s'\n",buf);

    if(TEST_DIR(buf))
      goto RETURN_OK;
    else if(mksubdir)
    {
      if(CREATE_DIR(buf))
        goto RETURN_OK;
    }
DIR_FAIL: ;
  }

  return(NULL); /* Couldn't find any data directories */

RETURN_OK:
  strncpy(bufout,buf,maxlen-2);
  bufout[maxlen-1]='\0';
  strcat(bufout,DIR_SEP_STR);
  return(bufout);
}

/*! \brief Prints paths to stderr */
void SDL_DirPrint()
{
  int n,m;
  const char *prefix[]={"Static","Dynamic",NULL};
  const char **lists[]={static_data_path,dynamic_data_path,NULL};

  for(m=0; prefix[m]!=NULL; m++)
  {
    fprintf(stderr,"%s data path:\n",prefix[m]);
    if(lists[m][0] == NULL)
      fprintf(stderr,"\tNONE!\n");

    for(n=0; lists[m][n]!=NULL; n++)
    {
      fprintf(stderr,"\t%s",lists[m][n]);
      if(lists[m][n][0] == '$')
        fprintf(stderr," = %s",getenv(lists[m][n]+1));
      fprintf(stderr,"\n");
    }
    fprintf(stderr,"\n");
  }
}
/**
 * Dynamic data paths are the search paths for data that that can be 
 * written to, generally to preserve settings between invocations.
 * Per-user on multiuser systems.
 */
const char *dynamic_data_path[] =
  {
    "$SDL_DDATA_PATH",

#ifdef WCE_PATHS
    "/",
#endif/*WCE_PATHS*/

#ifdef WIN32_PATHS
  "c:\\Documents and Settings\\$username\\Local Settings\\Application Data\\",
  ".\\",
#endif/*WIN32_PATHS*/

#ifdef UNIX_PATHS
    "$HOME/",
    "./",
#endif/*UNIX_PATHS*/

    NULL
  };

/**
 * Static data paths are paths pointing to common, read-only data for a 
 * program -- graphics, music, so forth.
 */
const char *static_data_path[] =
  {
    "$SDL_SDATA_PATH",

#ifdef WCE_PATHS
    "/",
#endif/*WCE_PATHS*/

#ifdef WIN32_PATHS
	"c:\\Documents and Settings\\$username\\Application Data\\",
    "c:\\windows\\application data\\",
    "c:\\program files\\",
    ".\\",
#endif/*WIN32_PATHS*/

#ifdef UNIX_PATHS
    "/usr/share/games/",
    "/usr/share/",
    "/usr/local/share/games/",
    "/usr/local/share/",
    "./",
#endif

    NULL /* end of list */
  };

